#include <Servo.h>                // biblioteca pentru servomotor
#include <Wire.h>                 // bibliotecă pentru dispozitivele I2C
#include <LiquidCrystal_I2C.h>    // bibliotecă pentru LCD
#include "pitches.h"

// se realizează conectarea LCD-ului prin I2C folosind adresa modului I2C: 0x27
// și se specifică faptul că e un lcd cu 2 linii, fiecare linie de 16 caractere 
LiquidCrystal_I2C lcd(0x27, 16, 2);

// se crează un obiect servo pentru a controla servomotorul
Servo servo1;

const int buzzer = 11;
const int servo1_pin = 10;

const int ultras_trig_pin = 9;
const int ultras_echo_pin = 8;

const int button = 2;

long duration, distance;
long average[3];

int count_openings, button_state;

// se definește numărul de deschideri după care se preconizează că trebuie dus gunoiul
const int max_op = 5;

// notele din cântec
int song[] = {
  NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3, 0, NOTE_B3, NOTE_C4
};

// duratele notelor
int note_durations[] = {
  4, 8, 8, 4, 4, 4, 4, 4
};

void setup() {

    // se pornește rezistența de pull-up
    pinMode(button, INPUT_PULLUP);

    // cu această variabilă se contorizează de câte ori s-a introdus ceva în coș
    count_openings = 0;

    // se inițializează lcd-ul
    lcd.init();
    lcd.noBacklight();

    pinMode(buzzer, OUTPUT);
  
    servo1.attach(servo1_pin);

    // se setează brațul servomotorului la 0 grade
    servo1.write(0);
  
    delay(200);
    
    pinMode(ultras_trig_pin, OUTPUT);  
    pinMode(ultras_echo_pin, INPUT);
} 

void measure() {  
     // se oprește pin-ul trig în cazul în care a fost pornit înainte.
    digitalWrite(ultras_trig_pin, LOW);
    delayMicroseconds(2);

    // se pornește pin-ul și se trimite o undă sonoră
    // se așteaptă 10 micro secunde pentru ca operația să se întâmple și apoi se oprește pinul
    digitalWrite(ultras_trig_pin, HIGH);
    delayMicroseconds(10);
    digitalWrite(ultras_trig_pin, LOW);

    //se extrage durata pulsului
    duration = pulseIn(ultras_echo_pin, HIGH);

    //se calculeaza distanta folosind formula: Distance = (Time x Sound speed in Air (340 m/s))/2
    //se calculeaza in cm : ((340 * 100) / 1000000)) = 0.034
    distance = (duration * 0.034) / 2;
}

void play() {

  // se iterează printre notele melodiei
  for (int i = 0; i < 8; i++) {

    // se calculează durata notei
    int note_duration = 1000 / note_durations[i];

    tone(buzzer, song[i], note_duration);

    // pentru a distinge notele, se setează un timp între ele
    // pentru a avea aprox. 3s coșul deschis se înmulțesc valorile cu  1.8 
    int pause = note_duration * 1.8;

    delay(pause);

    noTone(buzzer);
   
   }
  
}

void loop() {

  // se verifica starea butonului
  button_state = digitalRead(button);

  // când butonul a fost apăsat înseamnă că a fost golit coșul
  if(button_state == LOW) {

    count_openings = 0;

    tone(buzzer, 570, 800);

    lcd.backlight();
    lcd.setCursor(3, 0);
    lcd.print("The bin is");
    lcd.setCursor(2, 1);
    lcd.print("empty now! :D");

    delay(5000);

    noTone(buzzer);
    lcd.clear();
    lcd.noBacklight();

  } else {

    // se calculeaza o distanță medie realizând 3 măsurători
    for(int i = 0; i <= 2; i++) {
      measure();
      average[i] = distance;
  
      //delay între măsurători
      delay(10);
    }
    
    distance = (average[0] + average[1] + average[2]) / 3;
  
    // se verifică dacă un obiect se apropie de coșul de gunoi
    if(distance < 40) {
  
      if(count_openings >= 3) {
  
        servo1.attach(servo1_pin);
        delay(1);
  
        // pentru a aprinde lumina de pe fundalul lcd-ului
        lcd.backlight();
        lcd.setCursor(2, 0);
        lcd.print("Take out the");
        lcd.setCursor(4, 1);
        lcd.print("trash! :D");
  
        servo1.write(180);
        play();

        // se așteaptă mai mult în cazul în care se realizează golirea coșului
        delay(8000);
        servo1.write(0);
        lcd.clear();
        lcd.noBacklight();
          
      } else {
  
        //se atașează variabila servo1 la pin-ul corespunzător
        servo1.attach(servo1_pin);
        
        delay(1);
  
        lcd.backlight();
        
        // se afișează un mesaj când coșul se deschide
        lcd.setCursor(4, 0);
        lcd.print("Hello! :)");
    
        // se porneste buzzer-ul care anunță deschiderea coșului
        tone(buzzer, 1250, 700);
    
        // se rotește brațul servomotorului la 180 de grade
        servo1.write(180);
    
        delay(3000);
    
        // se curăță ecranul
        lcd.clear();
    
        // se afișează un mesaj când se închide coșul
        lcd.setCursor(4,0);
        lcd.print("Goodbye!"); 
    
        //se oprește buzzer-ul
        noTone(buzzer);
        
        // brațul servomotorului revine la 0 grade
        servo1.write(0);
        
        delay(2000);
    
        // se detașează servo-ul de la pin-ul acestuia
        servo1.detach();
    
        lcd.clear();
        lcd.noBacklight();
      }
  
      count_openings++;
      
    }
  }
}
